/*=======================================================================*/
/*================================ COPY ================================*/
/*=======================================================================*/
function copyToRight() {
  copyGraph(graph, transfGraph, transfCanvas);
  setNewRightFragmentSize(fragSize);
}
function copyToLeft() {
  copyGraph(transfGraph, graph, canvas);
  setNewFragmentSize(right_fragSize); 
  stateHandler();
}
function copyGraph(fromGraph, toGraph, toCanvas) {
  if (fromGraph.origin.length != []) {
    toGraph.origin = new Coordinate();
    toGraph.origin.copy(fromGraph.origin);
  }
  
  toGraph.axes = [];
  for (let i = 0; i < fromGraph.axes.length; i++) {
    toGraph.axes.push(new Coordinate());
    toGraph.axes[i].copy(fromGraph.axes[i]);
  }
  
  toGraph.lattice = [];
  for (let i = 0; i < fromGraph.lattice.length; i++) {
    toGraph.lattice.push(new Coordinate());
    toGraph.lattice[i].copy(fromGraph.lattice[i]);
  }
  
  toGraph.vertices = [];
  toGraph.vorbits = [];
  toGraph.vshifts = [];
  for (let i = 0; i < fromGraph.vertices.length; i++) {
    toGraph.vertices.push(new Coordinate());
    toGraph.vshifts.push(new Coordinate());
    toGraph.vertices[i].copy(fromGraph.vertices[i]);
    toGraph.vshifts[i].copy(fromGraph.vshifts[i]);
    toGraph.vorbits.push(fromGraph.vorbits[i]);
  }
  
  toGraph.vertexRepsFrac = [];
  for (let i = 0; i < fromGraph.vertexRepsFrac.length; i++) {
    toGraph.vertexRepsFrac.push(new Coordinate());
    toGraph.vertexRepsFrac[i].copy(fromGraph.vertexRepsFrac[i]);
    if (toGraph.vertexRepsFrac.length > VERTEX_COLORS.length) {
      //https://stackoverflow.com/questions/1152024/best-way-to-generate-a-random-color-in-javascript
      var randomColor = '#' + (0x1000000 + Math.floor(Math.random() * 0x1000000)).toString(16).substr(1);
      VERTEX_COLORS.push(randomColor);
    }
  }
  
  toGraph.edges = [];
  toGraph.eorbits = [];
  for (let i = 0; i < fromGraph.edges.length; i++) {
    toGraph.edges.push([fromGraph.edges[i][0], fromGraph.edges[i][1]]);
    toGraph.eorbits.push(fromGraph.eorbits[i]);
  }
  
  toGraph.edgeReps = [];
  toGraph.eRepShifts = [];
  for (let i = 0; i < fromGraph.edgeReps.length; i++) {
    toGraph.edgeReps.push([fromGraph.edgeReps[i][0], fromGraph.edgeReps[i][1]]);
    toGraph.eRepShifts.push(new Coordinate());
    toGraph.eRepShifts[i].copy(fromGraph.eRepShifts[i]);
    if (EDGE_COLORS.length < toGraph.edgeReps.length)
          EDGE_COLORS.push('#' + (0x1000000 + Math.floor(Math.random() * 0x1000000)).toString(16).substr(1));
  }
  toGraph.faces = [];
  toGraph.forbits = [];
  
  for (let i = 0; i < fromGraph.faces.length; i++) {
    toGraph.faces.push(fromGraph.faces[i]);
    toGraph.forbits.push(fromGraph.forbits[i]);
  }
  toGraph.computeSpan();

  drawGraph(toCanvas, toGraph, settings);
}
/*=======================================================================*/
/*================================ RESET ================================*/
/*=======================================================================*/

function resetAll(canvas, graph, transfCanvas, transfGraph, settings, deviceInput) {
  settings.reset();
  reset(canvas, graph, settings, deviceInput);
  transfGraph.reset();
  updateColors(transfGraph);
  drawGraph(transfCanvas, transfGraph, settings);
}
function reset(canvas, graph, settings, deviceInput) {
  //Reset graph fields
  graph.reset();
  updateColors(graph);
  
  //Set state to Origin
  var states = document.getElementsByName('states');
  enableDisableAllButtons(states, true);
  states[0].disabled = false;
  states[0].checked = true;
  
  //Reset sliders to default values
  setNewFragmentSize(new FragmentSize(SLIDER_DEFAULT_RANGE));
  
  //Reset graph title
	document.getElementById("graphTitle").value = "";
  
  //Reset device variable. Draw and print new graph information
  resetSelected(deviceInput);
  drawGraph(canvas, graph, settings);
  stateHandler();
	printGraph(graph);
}

function resetSelected(deviceInput) {
	deviceInput.previousSelected = deviceInput.selected = -1;
}

function resetFaces(graph, canvas) {
  graph.resetFInfo();
  drawGraph(canvas, graph, settings);
  stateHandler();
	printGraph(graph);
}
/*============================================================================*/
/*================================ INITIALIZE ================================*/
/*============================================================================*/
function getExampleStr() {
	var exampleStr = '{"sqfgraph":[\
	{"axes":[\
		{"xaxis":[417.06,-216.78]},\
		{"yaxis":[-357.03,-300.7]}\
	]},\
	{"fpts":[\
		[0.49,0.35],\
		[0.11,0.57]\
	]},\
	{"edges":[\
		[{"edge":[1,0]},{"shift":[0,1]}],\
		[{"edge":[0,1]},{"shift":[0,0]}],\
		[{"edge":[1,0]},{"shift":[-1,0]}]\
	]}\
]}'
  return exampleStr;
}
/*========================================================================*/
/*================================ UPLOAD ================================*/
/*========================================================================*/

function loadExample(canvas, graph, transfCanvas, transfGraph, settings, deviceInput) {
  var origin;
  if (graph.origin === null) {
    origin = new Coordinate(1000,1000);
  }
  else
    origin = graph.origin;
  resetAll(canvas, graph, transfCanvas, transfGraph, settings, deviceInput);
  
  loadGraph(graph, getExampleStr(), origin, 'sqf');
  drawGraph(canvas, graph, settings);
  printGraph(graph);

  drawGraph(canvas, graph, settings);
}

function uploadGraph(canvas, graph, settings, upload, uploadFileElement, format) {
  if (typeof(upload.files[0]) !== "undefined") {
    var origin;
    if (graph.origin === null) {
      origin = new Coordinate(1000,1000);
    }
    else
      origin = graph.origin;
    /*resetAll(canvas, graph, transfCanvas, transfGraph, settings, deviceInput);
    if (graph.origin === null) {
      alert("Please select an origin first before uploading.");
      return;
    }
    var origin = new Coordinate(graph.origin.x, graph.origin.y);*/
    reset(canvas, graph, settings, deviceInput);
    
    let uploadFile = upload.files[0];
    const fileReader = new FileReader();
    fileReader.onload = (event) => {
      var fileString = event.target.result;
      loadGraph(graph, fileString, origin, format);
      drawGraph(canvas, graph, settings);
      printGraph(graph);
    }
    fileReader.readAsText(uploadFile);
  }
  drawGraph(canvas, graph, settings);
  //to reset the value of the uploadFile input. If this is not done you cannot upload the same file back to back
  document.getElementById(uploadFileElement).value = null;
}

function loadGraph(graph, fileString, origin, format) {
  var obj = JSON.parse(fileString);
  var keys = Object.keys(obj);
  if (keys.length > 0) {
    if(format === 'sqf' && obj.sqfgraph) {
      parseSqfJson(graph, obj.sqfgraph, origin);
    }
    else if(format === 'fsf'&& obj.fsfgraph) {
      parseFsfJson(graph, obj.fsfgraph, origin);
    }
    else
      alert("Upload Failed: file must contain either an SQF or an FSF object in json format.");
  }
  else {
    alert("Upload Failed: JSON keys could not be detected.");
  }
  
  document.querySelector('input[name="states"]:checked').value = "edges";
  var states = document.getElementsByName('states');
  enableDisableAllButtons(states, false);
  states[4].checked = true;
  stateHandler();
}

/*==========================================================================*/
/*================================ DOWNLOAD ================================*/
/*==========================================================================*/

function downloadGraph(graph, format) {
	var encodedGraph;
  var filename;
  var title;
  if (format === 'sqf')
    encodedGraph = graphToSqfJson(graph);
  else
    encodedGraph = graphToFsfJson(graph);

  title = document.getElementById("graphTitle").value;
  if (title === "")
    title = "Untitled";
	filename = title + "." + format;
	let downloadElement = document.createElement('a');
	downloadElement.setAttribute('href', `data:text/` + format + `;charset=utf-8,${encodeURIComponent(encodedGraph)}`);
	downloadElement.setAttribute('download', filename); 
	downloadElement.style.display = 'none';
	document.body.appendChild(downloadElement);
	downloadElement.click();
	document.body.removeChild(downloadElement);
}
