/*================================================================================*/
/*================================= MAIN PARSERS =================================*/
/*================================================================================*/
//The shifted quotient (fractional) graph format includes a single edge copy for each edge orbit. For a vertex orbit, there may be more than 1 vertex copy. The axes vectors are also included.
function parseSqfJson(graph, sqfObj, origin) {
  graph.reset();
  graph.origin = origin;
  //*****TODO: you need to set the frac size somewhere
  
  parseAxesInfo(graph, sqfObj[0].axes);
  parseRepVertexFracInfo(graph, sqfObj[1].fpts);
  parseRepEdgeInfo(graph, sqfObj[2].edges);
  //console.log("fragSize: " + fragSize);
  //var newSize = computeMinFragmentSize(graph);
  var newSize = new FragmentSize(SLIDER_DEFAULT_RANGE);
  setNewFragmentSize(newSize);
  graph.makeNewFragment(fragSize);
  graph.computeSpan();
  graph.computeLattice(fragSize);
}
//The fractional fragment graph format includes all vertices and edges drawn on the graphs. Orbit information for every edge and vertex is preserved along with representative vertices/edges. The fragment size and axes are also included.
function parseFsfJson(graph, fsfObj, origin) {
  graph.origin = origin;
  parseAxesInfo(graph, fsfObj[0].axes);
  parseFragInfo(graph, fsfObj[1].fragInterval); 
  parseVertexFracInfo(graph, fsfObj[2].fpts, fsfObj[3].vertexShifts, fsfObj[4].vertexOrbits);
  parseEdgeInfo(graph, fsfObj[5].edges, fsfObj[6].edgeOrbits);
  graph.computeSpan();
  graph.computeLattice(fragSize);
}
/*============================================================================*/
/*================================= FRAGMENT =================================*/
/*============================================================================*/
function parseFragInfo(graph, fragObj) {
  if (fragObj && fragObj.length === 2 && fragObj[0].length === 2 && fragObj[1].length === 2) {
    setNewFragmentSize(new FragmentSize([[parseInt(fragObj[0][0]),parseInt(fragObj[0][1])], [parseInt(fragObj[1][0]),parseInt(fragObj[1][1])]]));
  }
  else  {
    alert("Upload Failed: Missing fragment information. Please try again.");
    graph.reset(); 
    return;
  }
}
//used when uploading shifted quotient graphs
function computeMinFragmentSize(graph) {
  var maxx = SLIDER_MIN_RANGE[0][1];
  var minx = SLIDER_MIN_RANGE[0][0];
  var maxy = SLIDER_MIN_RANGE[1][1];
  var miny = SLIDER_MIN_RANGE[1][0];
  for (let i = 0; i < graph.eRepShifts.length; i++) {
    if (graph.eRepShifts[i].x > maxx)
      maxx = graph.eRepShifts[i].x;
    if (graph.eRepShifts[i].x < minx)
      minx = graph.eRepShifts[i].x;
    if (graph.eRepShifts[i].y > maxy)
      maxy = graph.eRepShifts[i].y;
    if (graph.eRepShifts[i].y < miny)
      miny = graph.eRepShifts[i].y;
  }
  //console.log("fragSize: " + fragSize);
  return (new FragmentSize([[minx,maxx], [miny,maxy]]));
}
/*========================================================================*/
/*================================= AXES =================================*/
/*========================================================================*/
function parseAxesInfo(graph, axesObj) {
  if (axesObj) {
    if (axesObj[0].xaxis) {
      if (axesObj[1].yaxis) {
        graph.axes.push(new Coordinate(parseFloat(axesObj[0].xaxis[0]), parseFloat(axesObj[0].xaxis[1])));
        graph.axes.push(new Coordinate(parseFloat(axesObj[1].yaxis[0]), parseFloat(axesObj[1].yaxis[1])));
      }
      else {
        alert("Upload Failed: Y-axis of the graph could not be found. Please try again.");
        graph.reset();
        return;
      }
    }
    else {
        alert("Upload Failed: X-axis of the graph could not be found. Please try again.");
        graph.reset();
        return;
      }
  }
  else {
    alert("Upload Failed: Axes of the graph could not be found. Please try again.");
    graph.reset();
    return;
  }
}
/*============================================================================*/
/*================================= VERTICES =================================*/
/*============================================================================*/
//Used for sqf parsing
function parseRepVertexFracInfo(graph, fptsObj) {
  var colors = [];
  if (fptsObj) {
    for (let i = 0; i < fptsObj.length; i++) {
      var fracCoord = new Coordinate(parseFloat(fptsObj[i][0]), parseFloat(fptsObj[i][1]));
      var cartCoord = fracToCartCoord(graph, fracCoord);
      graph.vertexRepsFrac.push(fracCoord);
      graph.vertices.push(cartCoord);
      graph.vshifts.push(new Coordinate(0,0));
      graph.vorbits.push(i);
      if (graph.vertexRepsFrac.length > VERTEX_COLORS.length) {
        //https://stackoverflow.com/questions/1152024/best-way-to-generate-a-random-color-in-javascript
        var randomColor = '#' + (0x1000000 + Math.floor(Math.random() * 0x1000000)).toString(16).substr(1);
        VERTEX_COLORS.push(randomColor);
      }
    }
  }
  else  {
    alert("Upload Failed: Missing vertex information. Please try again.");
    graph.reset(); 
    return;
  }
}
//Used for fsf parsing
function parseVertexFracInfo(graph, fptsObj, vshiftsObj, vorbitsObj) {
  if (fptsObj && vshiftsObj && vorbitsObj && (fptsObj.length === vshiftsObj.length) && (fptsObj.length === vorbitsObj.length)) {
    for (let i = 0; i < fptsObj.length; i++) {
      var fracCoord = new Coordinate(parseFloat(fptsObj[i][0]), parseFloat(fptsObj[i][1]));
      var cartCoord = fracToCartCoord(graph, fracCoord);
      graph.vertices.push(cartCoord);
      graph.vshifts.push(new Coordinate(parseInt(vshiftsObj[i][0]), parseFloat(vshiftsObj[i][1])));
      graph.vorbits.push(parseInt(vorbitsObj[i]));
    }
    graph.recomputeVertexReps();
  }
  else  {
    alert("Upload Failed: Missing vertex information. Please try again.");
    graph.reset(); 
    return;
  }
}
/*=========================================================================*/
/*================================= EDGES =================================*/
/*=========================================================================*/

//Used for sqf parsing
function parseRepEdgeInfo(graph, edgesObj) {
  var colors = [];
  if (edgesObj) {
    for (let i = 0; i < edgesObj.length; i++) {
      //if the vertices of the edge both lie in the unit cell
      if (parseInt(edgesObj[i][1].shift[0]) === 0 && parseInt(edgesObj[i][1].shift[1]) === 0) {
        graph.edges.push([parseInt(edgesObj[i][0].edge[0]), parseInt(edgesObj[i][0].edge[1])]);
      }
      //if one of the vertices lies outside -> we need to add it to the list of vertices
      else {
        var spanV_shift = new Coordinate(parseInt(edgesObj[i][1].shift[0]), parseInt(edgesObj[i][1].shift[1]));
        var spanV_orbit = parseInt(edgesObj[i][0].edge[1]);
        var spanV_coord = getTranslatedCopy(graph.vertices[spanV_orbit], spanV_shift, graph.axes);
        
        var inVList = false;
        var index = -1;
        for (let j = 0; j < graph.vorbits.length; j++)
          if (graph.vorbits[j] === spanV_orbit && shiftIsEqual(graph.vshifts[j], spanV_shift)) {
            inVList = true;
            index = j;
          }
        if (!inVList) {
          graph.vertices.push(spanV_coord);
          graph.vshifts.push(spanV_shift);
          graph.vorbits.push(spanV_orbit);
          graph.edges.push([parseInt(edgesObj[i][0].edge[0]), graph.vertices.length - 1]);
        }
        else {
          graph.edges.push([parseInt(edgesObj[i][0].edge[0]), index]);
        }
      }
      graph.eorbits.push(i);
      graph.edgeReps.push([parseInt(edgesObj[i][0].edge[0]), parseInt(edgesObj[i][0].edge[1])]);
      graph.eRepShifts.push(new Coordinate(parseInt(edgesObj[i][1].shift[0]), parseInt(edgesObj[i][1].shift[1])));
      if (EDGE_COLORS.length < graph.edgeReps.length)
        EDGE_COLORS.push('#' + (0x1000000 + Math.floor(Math.random() * 0x1000000)).toString(16).substr(1));
    }
  }
  else  {
    alert("Upload Failed: Missing edge information. Please try again.");
    graph.reset(); 
    return;
  } 
}
//Used for fsf parsing
function parseEdgeInfo(graph, edgesObj, edgeOrbitsObj) {
  if (edgesObj && edgeOrbitsObj && (edgesObj.length === edgeOrbitsObj.length)) {
    for (let i = 0; i < edgesObj.length; i++) {
      graph.edges.push([parseInt(edgesObj[i][0].edge[0]), parseInt(edgesObj[i][0].edge[1])]);
      graph.eorbits.push(parseInt(edgeOrbitsObj[i]));
    }
    graph.recomputeEdgeReps();
  }
  else  {
    alert("Upload Failed: Missing edge information. Please try again.");
    graph.reset(); 
    return;
  } 
}