/*===========================================================================================*/
/*========================================== GRAPH ==========================================*/
/*===========================================================================================*/
//Shifted Quotient Graph (Fractional)
function graphToSqfJson(graph) {
  var str = '{"sqfgraph":[';
  if (graph.axes.length === 2) { 
    str += axesToJson(graph.axes);
    if (graph.vertexRepsFrac.length > 0) {
      str += ',' + vertexRepsToJson(graph.vertexRepsFrac);
      if (graph.edgeReps.length > 0)
        str += ',' + edgeRepsToJson(graph.edgeReps, graph.eRepShifts);
    }
  }
  
  return str + '\n]}';
}
//Fragment Graph given with a fragment interval (Fractional)
function graphToFsfJson(graph) {
  var str = '{"fsfgraph":[';
  if (graph.axes.length === 2) { 
    str += axesToJson(graph.axes);
    if (fragSize) { 
      str += ',' + fragSizeToJson(fragSize);
      if (graph.vertices.length > 0) {
        str += ',' + verticesFracToJson(graph);
        str += ',' + vshiftsToJson(graph.vshifts);
        str += ',' + vorbitsToJson(graph.vorbits);
        if (graph.edges.length > 0) {
          str += ',' + edgesToJson(graph.edges, graph.eorbits, graph.eRepShifts);
          str += ',' + eorbitsToJson(graph.eorbits);
        }
      }
    }
  }
  return str + '\n]}';
}

/*============================================================================================*/
/*==================================== ORIGIN, AXES, FRAG ====================================*/
/*============================================================================================*/

//NOT IN USE
function originToJson(origin) {
  return '{"origin":[' + Math.round(origin.x) + ',' + Math.round(origin.y) + ']}';
}
function axesToJson(axes) {
  var xaxisStr = '\n\t\t{"xaxis":[' + (Math.round((axes[0].x + Number.EPSILON) * 100) / 100) + ',' +
                                      (Math.round((axes[0].y + Number.EPSILON) * 100) / 100) + ']}';
  var yaxisStr = '\n\t\t{"yaxis":[' + (Math.round((axes[1].x + Number.EPSILON) * 100) / 100) + ',' +
                                      (Math.round((axes[1].y + Number.EPSILON) * 100) / 100) + ']}';
  return '\n\t{"axes":[' + xaxisStr + ',' + yaxisStr + '\n\t]}';
}
function fragSizeToJson(fragmentInterval) {
  var fragStr = '\n\t{"fragInterval":[';
  fragStr += '\n\t\t[' + Math.round(fragmentInterval.minx) + ',' + Math.round(fragmentInterval.maxx) + '],';
  fragStr += '\n\t\t[' + Math.round(fragmentInterval.miny) + ',' + Math.round(fragmentInterval.maxy) + ']';
  return fragStr + '\n\t]}';
}

/*==================================================================================*/
/*==================================== VERTICES ====================================*/
/*==================================================================================*/

/*=================*/
/*== Vertex Reps ==*/
/*=================*/
function vertexRepToJson(vertexRep) {
  return '\n\t\t[' + (Math.round((vertexRep.x + Number.EPSILON) * 100) / 100) + ',' + 
                      (Math.round((vertexRep.y + Number.EPSILON) * 100) / 100) + ']';
}
function vertexRepsToJson(vertexReps) {
  var string = '\n\t{"fpts":[';
  for (let i = 0; i < vertexReps.length; i++) {
    string += vertexRepToJson(vertexReps[i]);
    if (i != vertexReps.length-1)
      string += ',';
  }
  return string + '\n\t]}';
}

/*==================*/
/*== All Vertices ==*/
/*==================*/
function vertexFracToJson(graph, vertex) {
  var fravC = cartToFracCoord(graph, vertex);
  return '\n\t\t[' + (Math.round((fravC.x + Number.EPSILON) * 100) / 100) + ',' + 
                      (Math.round((fravC.y + Number.EPSILON) * 100) / 100) + ']';
}
function verticesFracToJson(graph) {
  var string = '\n\t{"fpts":[';
  for (let i = 0; i < graph.vertices.length; i++) {
    string += vertexFracToJson(graph, graph.vertices[i]);
    if (i != graph.vertices.length-1)
      string += ',';
  }
  return string + '\n\t]}';
}
function vshiftsToJson(vshifts) {
  var string = '\n\t{"vertexShifts":[';
  for (let i = 0; i < vshifts.length; i++) {
    string += '\n\t\t[' + vshifts[i].x + ',' + vshifts[i].y + ']';
    if (i != vshifts.length-1)
      string += ',';
  }
  return string + '\n\t]}';
}
function vorbitsToJson(vorbits) {
  var string = '\n\t{"vertexOrbits":[';
  for (let i = 0; i < vorbits.length; i++) {
    string += vorbits[i];
    if (i != vorbits.length-1)
      string += ',';
  }
  return string + ']}';
}
/*===============================================================================*/
/*==================================== EDGES ====================================*/
/*===============================================================================*/
function edgeToJson(edgeRep, edgeShift) {
  var edgeStr = '{"edge":[' + edgeRep[0] + ',' + edgeRep[1] + ']}';
  var shiftStr = '{"shift":[' + edgeShift.x + ',' + edgeShift.y + ']}';
  return '\n\t\t[' + edgeStr + ',' + shiftStr + ']';
}
function edgeRepsToJson(edgeReps, eRepShifts) {
  var string = '\n\t{"edges":[';
  for (let i = 0; i < edgeReps.length; i++) {
    string += edgeToJson(edgeReps[i], eRepShifts[i]);
    if (i != edgeReps.length-1)
      string += ',';
  }
  return string + '\n\t]}';
}
function edgesToJson(edges, eorbits, eRepShifts) {
  var string = '\n\t{"edges":[';
  for (let i = 0; i < edges.length; i++) {
    string += edgeToJson(edges[i], eRepShifts[eorbits[i]]);
    if (i != edges.length-1)
      string += ',';
  }
  return string + '\n\t]}';
}
function eorbitsToJson(eorbits) {
  var string = '\n\t{"edgeOrbits":[';
  for (let i = 0; i < eorbits.length; i++) {
    string += eorbits[i];
    if (i != eorbits.length-1)
      string += ',';
  }
  return string + ']}';
}

